/**This source contains some operations with complex values and angles. */
#ifndef __Angle_FB_h__
#define __Angle_FB_h__
#include <string.h>  //memset
#include <math.h>
#include <applstdefJc.h>

/**calculation factor from rad to int32-angle: It is 180 degree / M_PI = 2147483648.0f / M_PI */
#define krPi_int32Angle 0x28BE60DC

#define M_PI 3.141592653f
#define k30_int32Angle 0x15555555
#define k60_int32Angle 0x2aaaaaaa
#define k90_int32Angle 0x40000000
#define k120_int32Angle 0x55555555
#define k249_int32Angle 0xaaaaaaaa



/**Conversion from 16-bit-angle representation to float value in radiant.
 * @param VAL16 angle between -0x8000 (= 180 ) and 0x7fff (= 179.99)
 * @return angle between -M_PI (=180) and M_PI-0.000001 (= 179.999)
 * @Implementation notes: The VDSP-Processor stores a 16-bit-value into 32 bit.
 *      The representation of the int16-value may have a expansion with 0xffff0000 if the sign is negative or not.
 *      It depends from the machine code instruction usage. The casting from (int16) to (int32) or (float)
 *      doesn't work properly. Therefore the value is shifted to the int32 hi-position and then multiplied.
 *      It is a special behavior of that processor. But the macro works at all platforms.
 */
#define int16_to_rad(VAL) (( (float)(((int32)(VAL))<<16) ) * (M_PI / 2147483648.0f))


/**Conversion from 32-bit-angle representation to float value in grad.
 * @param VAL32 angle between -0x80000000 (= 180 ) and 0x7fffffff (= 179.99)
 * @return angle between -180 and  179.999
 */
#define int32_to_grad(VAL32) (( (float)(((int32)(VAL32))) ) * (180.0f / 2147483648.0f))


/**Conversion from 32-bit-angle representation to float value in rad.
 * @param VAL32 angle between -0x80000000 (= 180 ) and 0x7fffffff (= 179.99)
 * @return angle between -M_PI (=180) and M_PI-0.000001 (= 179.999)
 */
#define int32_to_rad(VAL32) (( (float)(((int32)(VAL32))) ) * (M_PI / 2147483648.0f))


/**Conversion from float value in radiant to 16-bit-angle representation.
 * @param RAD angle between -M_PI (=180) and M_PI-0.000001 (= 179.999) with possible overdrive (modulo 2*M_PI).
 * @return angle between -0x8000 (= 180 ) and 0x7fff (= 179.99)
 * @Implementation notes: The higher bits if overdriven are removed. It is a general solution.
 */
#define rad_to_int16(RAD) ((int16)( (RAD) * (32768.0f / M_PI))) 


/**Conversion from float value in radiant to 32-bit-angle representation.
 * Reards radiant values from approximately -350..350.
 * Implementation note: 24 bits are used to convert float to integer, The result is shifted 8 bits to left.
 * @param RAD angle between -M_PI (=180) and M_PI-0.000001 (= 179.999) with possible overdrive (modulo 2*M_PI).
 * @return angle between -0x80000000 (= 180 ) and 0x7fffffff (= 179.99)
 */
#define rad_to_int32(RAD) (((int32)( (RAD) * (8388608.0f / M_PI))<<8)) 

/**Conversion from float value in grad to 32-bit-angle representation.
 * @param RAD angle between -M_PI (=180) and M_PI-0.000001 (= 179.999) with possible overdrive (modulo 2*M_PI).
 * @return angle between -0x80000000 (= 180 ) and 0x7fffffff (= 179.99)
 */
#define grad_to_int32(GRAD) ((int32)( (GRAD) * (2147483648.0f / 180.0f))) 

#define grad_to_rad(GRAD) ((GRAD) * (M_PI / 180.0f))

#define rad_to_grad(RAD) ((RAD) * (180.0f / M_PI))
#define mult_complex_FB(x1, x2, y) \
{ y.re = x1.re * x2.re - x1.im * x2.im; \
  y.im = x1.im * x2.re + x1.re * x2.im; \
}



/**This structure describes a complex signal value with its angle and magnitude.
 * @simulink bus
 */
typedef struct Angle_abwmf_FB_t
{ 
  union { int32 _obj_[6]; ObjectJc obj; };

  /**rotating normal vector (magnitude = 1) for the given angle. */
  float_complex ab;
 
  /**revers rotating normal vector (magnitude = 1) for the given angle. The im-Component is negative. */
  float_complex anb;

  /**Angle in int32. */
  int32 wi;

  /**Angle in radiant. */
  float wrad;

  /**Magnitude of a signal. It is 1.0 if only an angle is given. */
  float m;

  /**Frequency of the signal. It should be a middle value of the angle difference per step. */
  float fq;

} Angle_abwmf_FB;


extern_C const struct ClassJc_t reflection_Angle_abwmf_FB;


/**sets the angle values with given integer angle. 
 * The output bus from simulink will be allocated in simulink respectively in the generated code.
 * Use the pointer to the existing output data, that are the instance data.
 * @param input the angle
 * @simulink ctor
 */
METHOD_C void ctor_Angle_abwmf_FB(Angle_abwmf_FB* thiz, int32 identObj);


/**sets the angle values with given integer angle. 
 * The output bus from simulink will be allocated in simulink respectively in the generated code.
 * Use the pointer to the existing output data, that are the instance data.
 * @param input the angle
 * @param fq associated frequency parameter.
 * @simulink Object-FB, accel-tlc
 */
INLINE_Fwc void set_Angle_abwmf_FB(Angle_abwmf_FB* thiz, int32 wi, float fq, float m, float* wrad_y) //, float_complex* ab_y)
{
#ifndef __reflectionHidden__ 
   thiz->wi = wi;
   float wrad = int32_to_rad(wi);
   thiz->wrad = wrad;
   thiz->ab.re = thiz->anb.re = cosf(wrad);
   thiz->anb.im = - (thiz->ab.im = sinf(wrad));
   thiz->fq = fq;
   thiz->m = m;
   *wrad_y = sizeof(void*); //wrad;
#endif
}










#endif //__Angle_FB_h__
